@@ -40,7 +40,7 @@ Data parsing mechanism differs depending on the data format type:
 """
 
 from collections import OrderedDict
-from io import BytesIO
+from io import BytesIO, TextIOWrapper
 import numpy as np
 import pandas
 from pandas.core.dtypes.cast import find_common_type
@@ -149,15 +149,36 @@ class PandasParser(object):
 
         Returns
         -------
-        pandas.Series
-            pandas.Series where index is columns names and values are
-            columns dtypes.
+        frame_dtypes : pandas.Series or dtype
+            Resulting dtype or pandas.Series where column names are used as
+            index and types of columns are used as values for full resulting
+            frame.
         """
-        return (
-            pandas.concat(cls.materialize(dtypes_ids), axis=1)
-            .apply(lambda row: find_common_type_cat(row.values), axis=1)
-            .squeeze(axis=0)
-        )
+        # each element in `partitions_dtypes` is a Series, where column names are
+        # used as index and types of columns for different partitions are used as values
+        partitions_dtypes = cls.materialize(dtypes_ids)
+        if all([len(dtype) == 0 for dtype in partitions_dtypes]):
+            return None
+
+        combined_part_dtypes = pandas.concat(partitions_dtypes, axis=1)
+        frame_dtypes = combined_part_dtypes.iloc[:, 0]
+
+        if not combined_part_dtypes.eq(frame_dtypes, axis=0).all(axis=None):
+            ErrorMessage.missmatch_with_pandas(
+                operation="read_*",
+                message="Data types of partitions are different! "
+                "Please refer to the troubleshooting section of the Modin documentation "
+                "to fix this issue",
+            )
+
+            # concat all elements of `partitions_dtypes` and find common dtype
+            # for each of the column among all partitions
+            frame_dtypes = combined_part_dtypes.apply(
+                lambda row: find_common_type_cat(row.values),
+                axis=1,
+            ).squeeze(axis=0)
+
+        return frame_dtypes
 
     @classmethod
     def single_worker_read(cls, fname, **kwargs):
@@ -210,28 +231,53 @@ class PandasCSVParser(PandasParser):
         num_splits = kwargs.pop("num_splits", None)
         start = kwargs.pop("start", None)
         end = kwargs.pop("end", None)
-        index_col = kwargs.get("index_col", None)
+        header_size = kwargs.pop("header_size", None)
+        encoding = kwargs.get("encoding", None)
         if start is not None and end is not None:
             # pop "compression" from kwargs because bio is uncompressed
-            bio = FileDispatcher.file_open(
+            with FileDispatcher.file_open(
                 fname, "rb", kwargs.pop("compression", "infer")
-            )
-            if kwargs.get("encoding", None) is not None:
-                header = b"" + bio.readline()
-            else:
+            ) as bio:
+                # In this case we beware that first line can contain BOM, so
+                # adding this line to the `header` for reading and then skip it
                 header = b""
-            bio.seek(start)
-            to_read = header + bio.read(end - start)
-            bio.close()
+                if encoding and (
+                    "utf" in encoding
+                    and "8" not in encoding
+                    or encoding == "unicode_escape"
+                    or encoding.replace("-", "_") == "utf_8_sig"
+                ):
+                    # do not 'close' the wrapper - underlying buffer is managed by `bio` handle
+                    fio = TextIOWrapper(bio, encoding=encoding, newline="")
+                    if header_size == 0:
+                        header = fio.readline().encode(encoding)
+                        kwargs["skiprows"] = 1
+                    for _ in range(header_size):
+                        header += fio.readline().encode(encoding)
+                elif encoding is not None:
+                    if header_size == 0:
+                        header = bio.readline()
+                        # `skiprows` can be only None here, so don't check it's type
+                        # and just set to 1
+                        kwargs["skiprows"] = 1
+                    for _ in range(header_size):
+                        header += bio.readline()
+                else:
+                    for _ in range(header_size):
+                        header += bio.readline()
+
+                bio.seek(start)
+                to_read = header + bio.read(end - start)
+
             pandas_df = pandas.read_csv(BytesIO(to_read), **kwargs)
         else:
             # This only happens when we are reading with only one worker (Default)
             return pandas.read_csv(fname, **kwargs)
-        if index_col is not None:
-            index = pandas_df.index
-        else:
-            # The lengths will become the RangeIndex
-            index = len(pandas_df)
+        index = (
+            pandas_df.index
+            if not isinstance(pandas_df.index, pandas.RangeIndex)
+            else len(pandas_df)
+        )
         return _split_result_for_readers(1, num_splits, pandas_df) + [
             index,
             pandas_df.dtypes,
@@ -292,6 +338,27 @@ class PandasCSVGlobParser(PandasCSVParser):
         ]
 
 
+@doc(_doc_pandas_parser_class, data_type="pickled pandas objects")
+class PandasPickleExperimentalParser(PandasParser):
+    @staticmethod
+    @doc(_doc_parse_func, parameters=_doc_parse_parameters_common)
+    def parse(fname, **kwargs):
+        warnings.filterwarnings("ignore")
+        num_splits = 1
+        single_worker_read = kwargs.pop("single_worker_read", None)
+        df = pandas.read_pickle(fname, **kwargs)
+        if single_worker_read:
+            return df
+        assert isinstance(
+            df, pandas.DataFrame
+        ), f"Pickled obj type: [{type(df)}] in [{fname}]; works only with pandas.DataFrame"
+
+        length = len(df)
+        width = len(df.columns)
+
+        return _split_result_for_readers(1, num_splits, df) + [length, width]
+
+
 @doc(_doc_pandas_parser_class, data_type="tables with fixed-width formatted lines")
 class PandasFWFParser(PandasParser):
     @staticmethod
@@ -505,7 +572,7 @@ class PandasExcelParser(PandasParser):
             has_index_names=is_list_like(header) and len(header) > 1,
             skiprows=skiprows,
             usecols=usecols,
-            **kwargs
+            **kwargs,
         )
         # In excel if you create a row with only a border (no values), this parser will
         # interpret that as a row of NaN values. pandas discards these values, so we
@@ -569,17 +636,6 @@ class PandasParquetParser(PandasParser):
     def parse(fname, **kwargs):
         num_splits = kwargs.pop("num_splits", None)
         columns = kwargs.get("columns", None)
-        if fname.startswith("s3://"):
-            from botocore.exceptions import NoCredentialsError
-            import s3fs
-
-            try:
-                fs = s3fs.S3FileSystem()
-                fname = fs.open(fname)
-            except NoCredentialsError:
-                fs = s3fs.S3FileSystem(anon=True)
-                fname = fs.open(fname)
-
         if num_splits is None:
             return pandas.read_parquet(fname, **kwargs)
         kwargs["use_pandas_metadata"] = True
