@@ -21,6 +21,7 @@ import pandas
 from pandas.core.dtypes.common import is_list_like
 from csv import QUOTE_NONE, Dialect
 import sys
+import io
 from typing import Union, Sequence, Callable, Dict, Tuple
 from pandas._typing import FilePathOrBuffer
 import pandas._libs.lib as lib
@@ -102,9 +103,6 @@ class CSVDispatcher(TextFileDispatcher):
             return cls.single_worker_read(filepath_or_buffer, **kwargs)
 
         is_quoting = kwargs.get("quoting", "") != QUOTE_NONE
-        quotechar = kwargs.get("quotechar", '"').encode(
-            encoding if encoding is not None else "UTF-8"
-        )
         # In these cases we should pass additional metadata
         # to the workers to match pandas output
         pass_names = names in [None, lib.no_default] and (
@@ -133,6 +131,12 @@ class CSVDispatcher(TextFileDispatcher):
         )
 
         with cls.file_open(filepath_or_buffer_md, "rb", compression_infered) as f:
+            old_pos = f.tell()
+            fio = io.TextIOWrapper(f, encoding=encoding, newline="")
+            newline, quotechar = cls.compute_newline(
+                fio, encoding, kwargs.get("quotechar", '"')
+            )
+            f.seek(old_pos)
             splits = cls.partitioned_file(
                 f,
                 num_partitions=NPartitions.get(),
@@ -140,6 +144,8 @@ class CSVDispatcher(TextFileDispatcher):
                 skiprows=skiprows_partitioning,
                 quotechar=quotechar,
                 is_quoting=is_quoting,
+                encoding=encoding,
+                newline=newline,
                 header_size=header_size,
                 pre_reading=pre_reading,
             )
@@ -158,7 +164,6 @@ class CSVDispatcher(TextFileDispatcher):
             column_names=column_names,
             skiprows_md=skiprows_md if should_handle_skiprows else None,
             header_size=header_size,
-            squeeze=kwargs.get("squeeze", False),
             skipfooter=kwargs.get("skipfooter", None),
             parse_dates=kwargs.get("parse_dates", False),
             nrows=kwargs.get("nrows", None) if should_handle_skiprows else None,
@@ -350,8 +355,6 @@ class CSVDispatcher(TextFileDispatcher):
                 new_query_compiler = new_query_compiler.view(
                     pandas.RangeIndex(len(new_query_compiler.index))[:nrows]
                 )
-        if kwargs.get("squeeze", False) and len(new_query_compiler.columns) == 1:
-            return new_query_compiler[new_query_compiler.columns[0]]
         if index_col is None:
             new_query_compiler._modin_frame.synchronize_labels(axis=0)
 
